/*
The software in this package is distributed under the GNU General
Public License version 2 (with a special exception described below).

A copy of GNU General Public License (GPL) is included in this distribution,
in the file COPYING.GPL.

As a special exception, if other files instantiate templates or use macros
or inline functions from this file, or you compile this file and link it
with other works to produce a work based on this file, this file
does not by itself cause the resulting work to be covered
by the GNU General Public License.

However the source code for this file must still be made available
in accordance with section (3) of the GNU General Public License.

This exception does not invalidate any other reasons why a work based
on this file might be covered by the GNU General Public License.
*/
#ifndef PING_INTERVAL_H
#define PING_INTERVAL_H

//-----------------------------------------------------------------------------
// PingIntervalType
//-----------------------------------------------------------------------------

typedef long PingIntervalType;

//-----------------------------------------------------------------------------
// PingInterval
//-----------------------------------------------------------------------------

/**
 * @brief Class designed to behave like a scalar type (i.e. int, long), but
 * with helper methods to encapsulate interval computation.
 * Scope: one object per host.
 */
class PingInterval
{
public:
    PingInterval( PingIntervalType interval );
    ~PingInterval();

    operator PingIntervalType() const;

    void back_to_original();
    void speed_up();

private:
    bool can_speed_up() const;

private:
    /// the original interval to ping, unchangeable
    const PingIntervalType OriginalInterval;
    /// the actual interval exported by this object
    PingIntervalType Interval;

};

#endif // PING_INTERVAL_H
